@@ -1,5 +1,7 @@
-/* 
-   Copyright (C) Andrew Tridgell 1998
+/* -*- c-file-style: "linux"; -*-
+   
+   Copyright (C) 1998-2001 by Andrew Tridgell <tridge@samba.org>
+   Copyright (C) 2001 by Martin Pool <mbp@samba.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,7 +18,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* the socket based protocol for setting up a connection wit rsyncd */
+/* the socket based protocol for setting up a connection with rsyncd */
 
 #include "rsync.h"
 
@@ -27,6 +29,10 @@ extern int rsync_port;
 char *auth_user;
 int sanitize_paths = 0;
 
+/*
+ * Run a client connected to an rsyncd.  The alternative to this
+ * function for remote-shell connections is do_cmd.
+ */
 int start_socket_client(char *host, char *path, int argc, char *argv[])
 {
 	int fd, i;
@@ -36,10 +42,30 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 	char *p, *user=NULL;
 	extern int remote_version;
 	extern int am_sender;
-	extern struct in_addr socket_address;
+	extern char *shell_cmd;
+	extern int kludge_around_eof;
+	extern char *bind_address;
+	extern int default_af_hint;
+       
+	if (argc == 0 && !am_sender) {
+		extern int list_only;
+		list_only = 1;
+	}
 
+        /* This is just a friendliness enhancement: if the connection
+         * is to an rsyncd then there is no point specifying the -e option.
+         * Note that this is only set if the -e was explicitly specified,
+         * not if the environment variable just happens to be set.
+         * See http://lists.samba.org/pipermail/rsync/2000-September/002744.html
+         */
+        if (shell_cmd) {
+                rprintf(FERROR, "WARNING: --rsh or -e option ignored when "
+                        "connecting to rsync daemon\n");
+                /* continue */
+        }
+        
 	if (*path == '/') {
-		rprintf(FERROR,"ERROR: The remote path must start with a module name\n");
+		rprintf(FERROR,"ERROR: The remote path must start with a module name not a /\n");
 		return -1;
 	}
 
@@ -53,7 +79,8 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 	if (!user) user = getenv("USER");
 	if (!user) user = getenv("LOGNAME");
 
-	fd = open_socket_out(host, rsync_port, &socket_address);
+	fd = open_socket_out_wrapped (host, rsync_port, bind_address,
+				      default_af_hint);
 	if (fd == -1) {
 		exit_cleanup(RERR_SOCKETIO);
 	}
@@ -82,6 +109,10 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 	io_printf(fd,"%s\n",path);
 	if (p) *p = '/';
 
+	/* Old servers may just drop the connection here,
+	 rather than sending a proper EXIT command.  Yuck. */
+	kludge_around_eof = remote_version < 25;
+
 	while (1) {
 		if (!read_line(fd, line, sizeof(line)-1)) {
 			return -1;
@@ -93,8 +124,12 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 		}
 
 		if (strcmp(line,"@RSYNCD: OK") == 0) break;
+
+		if (strcmp(line,"@RSYNCD: EXIT") == 0) exit(0);
+
 		rprintf(FINFO,"%s\n", line);
 	}
+	kludge_around_eof = False;
 
 	for (i=0;i<sargc;i++) {
 		io_printf(fd,"%s\n", sargs[i]);
@@ -117,7 +152,7 @@ static int rsync_module(int fd, int i)
 	char *argv[MAX_ARGS];
 	char **argp;
 	char line[MAXPATHLEN];
-	uid_t uid = (uid_t)-2;
+	uid_t uid = (uid_t)-2;  /* canonically "nobody" */
 	gid_t gid = (gid_t)-2;
 	char *p;
 	char *addr = client_addr(fd);
@@ -172,7 +207,7 @@ static int rsync_module(int fd, int i)
 		if (!name_to_uid(p, &uid)) {
 			if (!isdigit(*p)) {
 				rprintf(FERROR,"Invalid uid %s\n", p);
-				io_printf(fd,"@ERROR: invalid uid\n");
+				io_printf(fd,"@ERROR: invalid uid %s\n", p);
 				return -1;
 			} 
 			uid = atoi(p);
@@ -182,12 +217,19 @@ static int rsync_module(int fd, int i)
 		if (!name_to_gid(p, &gid)) {
 			if (!isdigit(*p)) {
 				rprintf(FERROR,"Invalid gid %s\n", p);
-				io_printf(fd,"@ERROR: invalid gid\n");
+				io_printf(fd,"@ERROR: invalid gid %s\n", p);
 				return -1;
 			} 
 			gid = atoi(p);
 		}
 	}
+        
+        /* TODO: If we're not root, but the configuration requests
+         * that we change to some uid other than the current one, then
+         * log a warning. */
+
+        /* TODO: Perhaps take a list of gids, and make them into the
+         * supplementary groups. */
 
 	p = lp_include_from(i);
 	add_exclude_file(p, 1, 1);
@@ -201,24 +243,36 @@ static int rsync_module(int fd, int i)
 	p = lp_exclude(i);
 	add_exclude_line(p);
 
-	log_open();
+	log_init();
 
 	if (use_chroot) {
+		/*
+		 * XXX: The 'use chroot' flag is a fairly reliable
+		 * source of confusion, because it fails under two
+		 * important circumstances: running as non-root,
+		 * running on Win32 (or possibly others).  On the
+		 * other hand, if you are running as root, then it
+		 * might be better to always use chroot.
+		 *
+		 * So, perhaps if we can't chroot we should just issue
+		 * a warning, unless a "require chroot" flag is set,
+		 * in which case we fail.
+		 */
 		if (chroot(lp_path(i))) {
-			rprintf(FERROR,"chroot %s failed\n", lp_path(i));
+			rsyserr(FERROR, errno, "chroot %s failed", lp_path(i));
 			io_printf(fd,"@ERROR: chroot failed\n");
 			return -1;
 		}
 
 		if (!push_dir("/", 0)) {
-			rprintf(FERROR,"chdir %s failed\n", lp_path(i));
+                        rsyserr(FERROR, errno, "chdir %s failed\n", lp_path(i));
 			io_printf(fd,"@ERROR: chdir failed\n");
 			return -1;
 		}
 
 	} else {
 		if (!push_dir(lp_path(i), 0)) {
-			rprintf(FERROR,"chdir %s failed\n", lp_path(i));
+			rsyserr(FERROR, errno, "chdir %s failed\n", lp_path(i));
 			io_printf(fd,"@ERROR: chdir failed\n");
 			return -1;
 		}
@@ -227,13 +281,13 @@ static int rsync_module(int fd, int i)
 
 	if (am_root) {
 		if (setgid(gid)) {
-			rprintf(FERROR,"setgid %d failed\n", gid);
+			rsyserr(FERROR, errno, "setgid %d failed", (int) gid);
 			io_printf(fd,"@ERROR: setgid failed\n");
 			return -1;
 		}
 
 		if (setuid(uid)) {
-			rprintf(FERROR,"setuid %d failed\n", uid);
+			rsyserr(FERROR, errno, "setuid %d failed", (int) uid);
 			io_printf(fd,"@ERROR: setuid failed\n");
 			return -1;
 		}
@@ -291,7 +345,8 @@ static int rsync_module(int fd, int i)
 		}
 	}
 
-	ret = parse_arguments(argc, argv, 0);
+        argp = argv;
+	ret = parse_arguments(&argc, (const char ***) &argp, 0);
 
 	if (request) {
 		if (*auth_user) {
@@ -306,22 +361,27 @@ static int rsync_module(int fd, int i)
 		free(request);
 	}
 
-#if !TRIDGE
+#ifndef DEBUG
 	/* don't allow the logs to be flooded too fast */
 	if (verbose > 1) verbose = 1;
 #endif
 
-	argc -= optind;
-	argp = argv + optind;
-	optind = 0;
-
 	if (remote_version < 23) {
 		if (remote_version == 22 || (remote_version > 17 && am_sender))
 			io_start_multiplex_out(fd);
 	}
+        
+        /* For later protocol versions, we don't start multiplexing
+         * until we've configured nonblocking in start_server.  That
+         * means we're in a sticky situation now: there's no way to
+         * convey errors to the client. */
 
+        /* FIXME: Hold off on reporting option processing errors until
+         * we've set up nonblocking and multiplexed IO and can get the
+         * message back to them. */
 	if (!ret) {
-		option_error();
+                option_error();
+                exit_cleanup(RERR_UNSUPPORTED);
 	}
 
 	if (lp_timeout(i)) {
@@ -340,10 +400,14 @@ static void send_listing(int fd)
 {
 	int n = lp_numservices();
 	int i;
-	
+	extern int remote_version;
+
 	for (i=0;i<n;i++)
 		if (lp_list(i))
 		    io_printf(fd, "%-15s\t%s\n", lp_name(i), lp_comment(i));
+
+	if (remote_version >= 25)
+		io_printf(fd,"@RSYNCD: EXIT\n");
 }
 
 /* this is called when a socket connection is established to a client
@@ -423,6 +487,7 @@ int daemon_main(void)
 	extern char *config_file;
 	extern int orig_umask;
 	char *pid_file;
+	extern int no_detach;
 
 	if (is_a_socket(STDIN_FILENO)) {
 		int i;
@@ -438,15 +503,21 @@ int daemon_main(void)
 		return start_daemon(STDIN_FILENO);
 	}
 
-	become_daemon();
+	if (!no_detach)
+	    become_daemon();
 
 	if (!lp_load(config_file, 1)) {
 		exit_cleanup(RERR_SYNTAX);
 	}
 
-	log_open();
+	log_init();
 
-	rprintf(FINFO,"rsyncd version %s starting\n",VERSION);
+	rprintf(FINFO, "rsyncd version %s starting, listening on port %d\n",
+		RSYNC_VERSION,
+                rsync_port);
+        /* TODO: If listening on a particular address, then show that
+         * address too.  In fact, why not just do inet_ntop on the
+         * local address??? */
 
 	if (((pid_file = lp_pid_file()) != NULL) && (*pid_file != '\0')) {
 		char pidbuf[16];
@@ -456,10 +527,10 @@ int daemon_main(void)
 		if ((fd = do_open(lp_pid_file(), O_WRONLY|O_CREAT|O_TRUNC,
 					0666 & ~orig_umask)) == -1) {
 		    cleanup_set_pid(0);
-		    rprintf(FLOG,"failed to create pid file %s\n", pid_file);
+		    rsyserr(FLOG, errno, "failed to create pid file %s", pid_file);
 		    exit_cleanup(RERR_FILEIO);
 		}
-		slprintf(pidbuf, sizeof(pidbuf), "%d\n", pid);
+		snprintf(pidbuf, sizeof(pidbuf), "%d\n", pid);
 		write(fd, pidbuf, strlen(pidbuf));
 		close(fd);
 	}
