@@ -1,5 +1,6 @@
-/* 
-   Copyright (C) Andrew Tridgell 1996
+/* -*- c-file-style: "linux" -*-
+   
+   Copyright (C) 1996-2000 by Andrew Tridgell 
    Copyright (C) Paul Mackerras 1996
    
    This program is free software; you can redistribute it and/or modify
@@ -35,6 +36,7 @@ extern int size_only;
 extern int io_timeout;
 extern int remote_version;
 extern int always_checksum;
+extern int modify_window;
 extern char *compare_dest;
 
 
@@ -54,7 +56,7 @@ static int skip_file(char *fname,
 
 		if (compare_dest != NULL) {
 			if (access(fname, 0) != 0) {
-				slprintf(fnamecmpdest,MAXPATHLEN,"%s/%s",
+				snprintf(fnamecmpdest,MAXPATHLEN,"%s/%s",
 						    compare_dest,fname);
 				fname = fnamecmpdest;
 			}
@@ -75,7 +77,7 @@ static int skip_file(char *fname,
 		return 0;
 	}
 
-	return (st->st_mtime == file->modtime);
+	return (cmp_modtime(st->st_mtime,file->modtime) == 0);
 }
 
 
@@ -175,6 +177,15 @@ static struct sum_struct *generate_sums(struct map_struct *buf,OFF_T len,int n)
 }
 
 
+
+/*
+ * Acts on file number I from FLIST, whose name is fname.
+ *
+ * First fixes up permissions, then generates checksums for the file.
+ *
+ * (This comment was added later by mbp who was trying to work it out;
+ * it might be wrong.)
+ */ 
 void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 {  
 	int fd;
@@ -199,7 +210,8 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 
 	if (only_existing && statret == -1 && errno == ENOENT) {
 		/* we only want to update existing files */
-		if (verbose > 1) rprintf(FINFO,"not creating %s\n",fname);
+		if (verbose > 1) rprintf(FINFO, RSYNC_NAME
+					 ": not creating new file \"%s\"\n",fname);
 		return;
 	}
 
@@ -213,10 +225,18 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 	}
 
 	if (S_ISDIR(file->mode)) {
-		if (dry_run) return;
+                /* The file to be received is a directory, so we need
+                 * to prepare appropriately.  If there is already a
+                 * file of that name and it is *not* a directory, then
+                 * we need to delete it.  If it doesn't exist, then
+                 * recursively create it. */
+          
+		if (dry_run) return; /* XXXX -- might cause inaccuracies?? -- mbp */
 		if (statret == 0 && !S_ISDIR(st.st_mode)) {
 			if (robust_unlink(fname) != 0) {
-				rprintf(FERROR,"unlink %s : %s\n",fname,strerror(errno));
+				rprintf(FERROR, RSYNC_NAME
+					": recv_generator: unlink \"%s\" to make room for directory: %s\n",
+                                        fname,strerror(errno));
 				return;
 			}
 			statret = -1;
@@ -225,11 +245,13 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 			if (!(relative_paths && errno==ENOENT && 
 			      create_directory_path(fname)==0 && 
 			      do_mkdir(fname,file->mode)==0)) {
-				rprintf(FERROR,"mkdir %s : %s (2)\n",
+				rprintf(FERROR, RSYNC_NAME ": recv_generator: mkdir \"%s\": %s (2)\n",
 					fname,strerror(errno));
 			}
 		}
-		if (set_perms(fname,file,NULL,0) && verbose) 
+		/* f_out is set to -1 when doing final directory 
+		   permission and modification time repair */
+		if (set_perms(fname,file,NULL,0) && verbose && (f_out != -1)) 
 			rprintf(FINFO,"%s/\n",fname);
 		return;
 	}
@@ -242,7 +264,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 
 		if (safe_symlinks && unsafe_symlink(file->link, fname)) {
 			if (verbose) {
-				rprintf(FINFO,"ignoring unsafe symlink %s -> %s\n",
+				rprintf(FINFO,RSYNC_NAME ": ignoring unsafe symlink \"%s\" -> \"%s\"\n",
 					fname,file->link);
 			}
 			return;
@@ -251,20 +273,26 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 			l = readlink(fname,lnk,MAXPATHLEN-1);
 			if (l > 0) {
 				lnk[l] = 0;
+				/* A link already pointing to the
+				 * right place -- no further action
+				 * required. */
 				if (strcmp(lnk,file->link) == 0) {
 					set_perms(fname,file,&st,1);
 					return;
 				}
-			}
+			}  
+			/* Not a symlink, so delete whatever's
+			 * already there and put a new symlink
+			 * in place. */			   
 			delete_file(fname);
 		}
 		if (do_symlink(file->link,fname) != 0) {
-			rprintf(FERROR,"symlink %s -> %s : %s\n",
+			rprintf(FERROR,RSYNC_NAME": symlink \"%s\" -> \"%s\": %s\n",
 				fname,file->link,strerror(errno));
 		} else {
 			set_perms(fname,file,NULL,0);
 			if (verbose) {
-				rprintf(FINFO,"%s -> %s\n",
+				rprintf(FINFO,RSYNC_NAME": %s -> %s\n",
 					fname,file->link);
 			}
 		}
@@ -297,12 +325,14 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 
 	if (preserve_hard_links && check_hard_link(file)) {
 		if (verbose > 1)
-			rprintf(FINFO,"%s is a hard link\n",f_name(file));
+			rprintf(FINFO, RSYNC_NAME
+				": \"%s\" is a hard link\n",f_name(file));
 		return;
 	}
 
 	if (!S_ISREG(file->mode)) {
-		rprintf(FINFO,"skipping non-regular file %s\n",fname);
+		rprintf(FINFO, RSYNC_NAME
+			": skipping non-regular file \"%s\"\n",fname);
 		return;
 	}
 
@@ -311,7 +341,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 	if ((statret == -1) && (compare_dest != NULL)) {
 		/* try the file at compare_dest instead */
 		int saveerrno = errno;
-		slprintf(fnamecmpbuf,MAXPATHLEN,"%s/%s",compare_dest,fname);
+		snprintf(fnamecmpbuf,MAXPATHLEN,"%s/%s",compare_dest,fname);
 		statret = link_stat(fnamecmpbuf,&st);
 		if (!S_ISREG(st.st_mode))
 			statret = -1;
@@ -327,7 +357,9 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 			if (!dry_run) send_sums(NULL,f_out);
 		} else {
 			if (verbose > 1)
-				rprintf(FERROR,"recv_generator failed to open %s\n",fname);
+				rprintf(FERROR, RSYNC_NAME
+					": recv_generator failed to open \"%s\": %s\n",
+					fname, strerror(errno));
 		}
 		return;
 	}
@@ -343,7 +375,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 		return;
 	}
 
-	if (update_only && st.st_mtime > file->modtime && fnamecmp == fname) {
+	if (update_only && cmp_modtime(st.st_mtime,file->modtime)>0 && fnamecmp == fname) {
 		if (verbose > 1)
 			rprintf(FINFO,"%s is newer\n",fname);
 		return;
@@ -370,7 +402,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 	fd = do_open(fnamecmp, O_RDONLY, 0);
 
 	if (fd == -1) {
-		rprintf(FERROR,"failed to open %s, continuing : %s\n",fnamecmp,strerror(errno));
+		rprintf(FERROR,RSYNC_NAME": failed to open \"%s\", continuing : %s\n",fnamecmp,strerror(errno));
 		/* pretend the file didn't exist */
 		write_int(f_out,i);
 		send_sums(NULL,f_out);
@@ -411,6 +443,11 @@ void generate_files(int f,struct file_list *flist,char *local_name,int f_recv)
 		rprintf(FINFO,"generator starting pid=%d count=%d\n",
 			(int)getpid(),flist->count);
 
+	/* we expect to just sit around now, so don't exit on a
+	   timeout. If we really get a timeout then the other process should
+	   exit */
+	io_timeout = 0;
+
 	for (i = 0; i < flist->count; i++) {
 		struct file_struct *file = flist->files[i];
 		mode_t saved_mode = file->mode;
@@ -421,6 +458,8 @@ void generate_files(int f,struct file_list *flist,char *local_name,int f_recv)
 		   them. This is then fixed after the files are transferred */
 		if (!am_root && S_ISDIR(file->mode)) {
 			file->mode |= S_IWUSR; /* user write */
+                        /* XXX: Could this be causing a problem on SCO?  Perhaps their
+                         * handling of permissions is strange? */
 		}
 
 		recv_generator(local_name?local_name:f_name(file),
@@ -438,11 +477,6 @@ void generate_files(int f,struct file_list *flist,char *local_name,int f_recv)
 
 	write_int(f,-1);
 
-	/* we expect to just sit around now, so don't exit on a
-	   timeout. If we really get a timeout then the other process should
-	   exit */
-	io_timeout = 0;
-
 	if (remote_version >= 13) {
 		/* in newer versions of the protocol the files can cycle through
 		   the system more than once to catch initial checksum errors */
