@@ -1,6 +1,7 @@
 /* 
-   Copyright (C) Andrew Tridgell 1996
+   Copyright (C) by Andrew Tridgell 1996, 2000
    Copyright (C) Paul Mackerras 1996
+   Copyright (C) 2001 by Martin Pool <mbp@samba.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,6 +18,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+
 #define False 0
 #define True 1
 
@@ -47,7 +49,18 @@
 #define SAME_TIME (1<<7)
 
 /* update this if you make incompatible changes */
-#define PROTOCOL_VERSION 24
+#define PROTOCOL_VERSION 25
+
+/* We refuse to interoperate with versions that are not in this range.
+ * Note that we assume we'll work with later versions: the onus is on
+ * people writing them to make sure that they don't send us anything
+ * we won't understand.
+ *
+ * There are two possible explanations for the limit at thirty: either
+ * to allow new major-rev versions that do not interoperate with us,
+ * and (more likely) so that we can detect an attempt to connect rsync
+ * to a non-rsync server, which is unlikely to begin by sending a byte
+ * between 15 and 30. */
 #define MIN_PROTOCOL_VERSION 15
 #define MAX_PROTOCOL_VERSION 30
 
@@ -63,7 +76,10 @@
 
 #define MPLEX_BASE 7
 
-enum logcode {FNONE=0, FERROR=1, FINFO=2, FLOG=3};
+/* Log values.  I *think* what these mean is: FLOG goes to the server
+ * logfile; FERROR and FINFO try to end up on the client, with
+ * different levels of filtering. */
+enum logcode {FNONE=0, FERROR=1, FINFO=2, FLOG=3 };
 
 #include "errcode.h"
 
@@ -77,12 +93,6 @@ enum logcode {FNONE=0, FERROR=1, FINFO=2, FLOG=3};
 
 #include <sys/types.h>
 
-#ifdef HAVE_GETOPT_LONG
-#include <getopt.h>
-#else
-#include "lib/getopt.h"
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -258,9 +268,11 @@ enum logcode {FNONE=0, FERROR=1, FINFO=2, FLOG=3};
 #endif
 
 #if HAVE_SHORT_INO_T
-#define INO_T uint32
+#  define INO_T uint32
+#elif HAVE_INO_T
+#  define INO_T ino_t
 #else
-#define INO_T ino_t
+#  define INO_T unsigned
 #endif
 
 #ifndef MIN
@@ -331,9 +343,9 @@ struct sum_buf {
 
 struct sum_struct {
   OFF_T flength;		/* total file length */
-  size_t count;			/* how many chunks */
-  size_t remainder;		/* flength % block_length */
-  size_t n;			/* block_length */
+  int count;			/* how many chunks */
+  int remainder;		/* flength % block_length */
+  int n;			/* block_length */
   struct sum_buf *sums;		/* points to info for each chunk */
 };
 
@@ -344,7 +356,6 @@ struct map_struct {
 };
 
 struct exclude_struct {
-	char *orig;
 	char *pattern;
 	int regular_exp;
 	int fnmatch_flags;
@@ -376,9 +387,28 @@ static inline int flist_up(struct file_list *flist, int i)
 }
 
 #include "byteorder.h"
-#include "version.h"
 #include "proto.h"
 #include "lib/mdfour.h"
+#include "lib/permstring.h"
+#include "lib/addrinfo.h"
+
+/* We have replacement versions of these if they're missing. */
+#ifndef HAVE_ASPRINTF
+int asprintf(char **ptr, const char *format, ...);
+#endif
+
+#ifndef HAVE_VASPRINTF
+int vasprintf(char **ptr, const char *format, va_list ap);
+#endif
+
+#if !defined(HAVE_VSNPRINTF) && !defined(HAVE_C99_VSNPRINTF)
+int vsnprintf (char *str, size_t count, const char *fmt, va_list args);
+#endif
+
+#if !defined(HAVE_SNPRINTF) && !defined(HAVE_C99_VSNPRINTF)
+int snprintf(char *str,size_t count,const char *fmt,...);
+#endif
+
 
 #if !HAVE_STRERROR
 extern char *sys_errlist[];
@@ -476,6 +506,13 @@ extern int errno;
 # define NONBLOCK_FLAG FNDELAY
 #endif
 
+#ifndef INADDR_LOOPBACK
+#define INADDR_LOOPBACK 0x7f000001
+#endif
+
+#ifndef INADDR_NONE
+#define INADDR_NONE 0xffffffff
+#endif
 
 #define IS_DEVICE(mode) (S_ISCHR(mode) || S_ISBLK(mode) || S_ISSOCK(mode) || S_ISFIFO(mode))
 
@@ -497,6 +534,14 @@ extern int errno;
 #endif
 ;
 
+/* This is just like rprintf, but it also tries to print some
+ * representation of the error code.  Normally errcode = errno. */
+void rsyserr(enum logcode, int, const char *, ...)
+#ifdef __GNUC__
+     __attribute__ ((format (printf, 3, 4)))
+#endif
+     ;
+
 #ifdef REPLACE_INET_NTOA
 #define inet_ntoa rep_inet_ntoa
 #endif
@@ -515,3 +560,15 @@ size_t strlcat(char *d, const char *s, size_t bufsize);
 #endif
 
 #define exit_cleanup(code) _exit_cleanup(code, __FILE__, __LINE__)
+
+
+extern int verbose;
+
+#ifndef HAVE_INET_NTOP
+const char *                 
+inet_ntop(int af, const void *src, char *dst, size_t size);
+#endif /* !HAVE_INET_NTOP */
+
+#ifndef HAVE_INET_PTON
+int isc_net_pton(int af, const char *src, void *dst);
+#endif
